@@ -168,18 +168,31 @@ class ResponseBody(object):
 
     def iterchunks(self):
         assert self.chunked
+        buffer = []
         while True:
+
             if self.resp.isclosed():
                 break
+
             chunksz = int(self.resp.fp.readline().strip(), 16)
             if not chunksz:
                 self.resp.fp.read(2) #crlf
                 self.resp.close()
                 self._release_conn()
                 break
+
             chunk = self.resp.fp.read(chunksz)
-            for ln in chunk.splitlines():
-                yield ln
+            for ln in chunk.splitlines(True):
+
+                end = ln == b'\n' and not buffer # end of response
+                if not ln or end:
+                    break
+
+                buffer.append(ln)
+                if ln.endswith(b'\n'):
+                    yield b''.join(buffer)
+                    buffer = []
+
             self.resp.fp.read(2) #crlf
 
 
@@ -349,6 +362,14 @@ class Session(object):
             if num_redirects > self.max_redirects:
                 raise RedirectLimit('Redirection limit exceeded')
             location = resp.getheader('location')
+            
+            # in case of relative location: add scheme and host to the location
+            location_split = util.urlsplit(location)
+
+            if not location_split[0]:
+                orig_url_split = util.urlsplit(url)
+                location = util.urlunsplit(orig_url_split[:2] + location_split[2:])
+
             if status == 301:
                 self.perm_redirects[url] = location
             elif status == 303:
